@@ -1,31 +1,22 @@
 #coding: utf-8
 
-from flask import request, jsonify
+import json
+from os import path, stat
+
+import numpy as np
+import pandas as pd
+from flask import jsonify, request, __version__ as flask_version
 from flask_restful import Resource
+from joblib import dump, load
+from sklearn import __version__ as skl_version
+from sklearn.impute import SimpleImputer
+from sklearn.metrics import accuracy_score, confusion_matrix
+from sklearn.preprocessing import LabelEncoder, StandardScaler
 from werkzeug.exceptions import BadRequest
 
-from joblib import load, dump
-
-import json
-
-from os import path, stat
-import sys
-
-import time
-
-import pandas as pd
-import numpy as np
-
-from . import __version__
-
-from .preprocessing import DatasetPreprocessing
+from . import __version__, logger
 from .estimator import EstimatorFactory
-
-
-from sklearn.preprocessing import LabelEncoder
-from sklearn.preprocessing import StandardScaler
-from sklearn.impute import SimpleImputer
-from sklearn.metrics import confusion_matrix, accuracy_score
+from .preprocessing import DatasetPreprocessing
 
 
 class ModelConfiguration():
@@ -42,13 +33,6 @@ class ModelConfiguration():
         self.features = self.__required_config('features')
         self.dependent = self.__required_config('dependent')
 
-        self.categoricals = self.__optional_config('categoricals')
-        if self.categoricals is None:
-            self.non_categoricals = self.features
-        else:
-            self.non_categoricals = [
-                item for item in self.features if item not in set(self.categoricals)]
-
         self.imputer = SimpleImputer(missing_values=np.nan, strategy='mean')
 
         self.standard_scaled = self.__optional_config('standard_scaled')
@@ -65,17 +49,18 @@ class ModelConfiguration():
         self.columns = None
         self.score = 0.0
         self.confusion = None
-        self.version = __version__
+        self.versions = self.actual_versions()
 
-    def __load_initial_config(self, name):
-        with open(ModelConfiguration.CONFIG_FILE % name, 'r') as configFile:
-            return json.load(configFile)
+    @staticmethod
+    def __load_initial_config(name):
+        with open(ModelConfiguration.CONFIG_FILE % name, 'r') as config_file:
+            return json.load(config_file)
 
     def __required_config(self, param):
         if param in self.__json_config:
             return self.__json_config[param]
         else:
-            raise Exception('Missing required param %s' % param)
+            raise RuntimeError('Missing required param %s' % param)
 
     def __optional_config(self, param, default=None):
         if param in self.__json_config:
@@ -83,9 +68,6 @@ class ModelConfiguration():
         else:
             return default
 
-    def has_categoricals(self):
-        return self.categoricals is not None
-
     def has_standard_scaled(self):
         return self.standard_scaled is not None
 
@@ -97,18 +79,27 @@ class ModelConfiguration():
             'name': self.name,
             'estimator_config': self.estimatorcfg,
             'features': self.features,
-            'categoricals': self.categoricals,
             'standard_scaled': self.standard_scaled,
             'dependent_encode': self.dependent_encode,
             'dependent': self.dependent,
             'score': self.score,
             'confusion': self.confusion,
-            'version': {
-                'model': self.version,
-                'service': __version__
+            'versions': {
+                'model': self.versions,
+                'running': self.actual_versions()
             }
         }
 
+    @staticmethod
+    def actual_versions():
+        return {
+            'learninghouse': __version__,
+            'scikit-learn': skl_version,
+            'pandas': pd.__version__,
+            'numpy': np.__version__,
+            'flask': flask_version
+        }
+
     def dump(self, estimator, columns, score, confusion):
         self.estimator = estimator
         self.columns = columns
@@ -135,8 +126,8 @@ class ModelAPI(Resource):
             return ModelAPI.make_json_response(modelcfg.config_object())
         except FileNotFoundError:
             return ModelAPI.make_json_response({}, 404, 'NOT_TRAINED')
-        except:
-            print("Unexpected error:", sys.exc_info()[0])
+        except Exception as exc:  # pylint: disable=broad-except
+            logger.exception(exc)
             return ModelAPI.make_json_response({}, 500, 'UNKNOWN_ERROR')
 
 
@@ -150,16 +141,18 @@ class ModelTraining(Resource):
             json_data = request.get_json()
             json_data = DatasetPreprocessing.add_time_information(json_data)
             if path.exists(filename):
-                df_temp = pd.read_csv(filename)
-                df = df_temp.append([json_data], ignore_index=True)
+                data_temp = pd.read_csv(filename)
+                data = data_temp.append([json_data], ignore_index=True)
             else:
-                df = pd.DataFrame([json_data])
+                data = pd.DataFrame([json_data])
 
-            df.to_csv(filename, sep=',', index=False)
+            data.to_csv(filename, sep=',', index=False)
 
-            return ModelTraining.train(model, df)
+            response = ModelTraining.train(model, data)
         else:
-            return ModelAPI.make_json_response({}, 400, 'BAD_REQUEST')
+            response = ModelAPI.make_json_response({}, 400, 'BAD_REQUEST')
+
+        return response
 
     @staticmethod
     def post(model):
@@ -167,43 +160,51 @@ class ModelTraining(Resource):
 
         if request.content_length == 0:
             if path.exists(filename):
-                df = pd.read_csv(filename)
-                return ModelTraining.train(model, df)
+                data = pd.read_csv(filename)
+                response = ModelTraining.train(model, data)
             else:
-                return ModelAPI.make_json_response({}, 202, 'NOT_ENOUGH_TRAINING_DATA')
+                response = ModelAPI.make_json_response(
+                    {}, 202, 'NOT_ENOUGH_TRAINING_DATA')
         else:
-            return ModelAPI.make_json_response({}, 400, 'BAD_REQUEST')
+            response = ModelAPI.make_json_response({}, 400, 'BAD_REQUEST')
+
+        return response
 
     @staticmethod
-    def train(model, df):
+    def train(model, data):
         try:
             modelcfg = ModelConfiguration(model)
 
-            if len(df.index) < 10:
+            if len(data.index) < 10:
                 return ModelAPI.make_json_response({}, 202, 'NOT_ENOUGH_TRAINING_DATA')
 
             modelcfg, x_train, x_test, y_train, y_test = DatasetPreprocessing.prepare_training(
-                modelcfg, df)
+                modelcfg, data)
 
-            estimator = EstimatorFactory.get_estimator(modelcfg.estimatorcfg)
+            estimator, modelcfg.estimatorcfg = EstimatorFactory.get_estimator(
+                **modelcfg.estimatorcfg)
 
             columns = x_train.columns
 
+            logger.debug('Train data columns: %s', columns)
+
             estimator.fit(x_train, y_train)
 
             y_pred = estimator.predict(x_test)
 
-            cm = confusion_matrix(y_test, y_pred)
+            confusion = confusion_matrix(y_test, y_pred)
             score = accuracy_score(y_test, y_pred)
 
-            modelcfg.dump(estimator, columns, score, cm.tolist())
+            modelcfg.dump(estimator, columns, score, confusion.tolist())
 
             return ModelAPI.make_json_response(modelcfg.config_object())
         except FileNotFoundError:
             return ModelAPI.make_json_response({}, 404, 'NO_CONFIGURATION')
-        # except:
-        #    print("Unexpected error:", sys.exc_info()[0])
-        #    return ModelAPI.make_json_response({}, 500, 'UNKNOWN_ERROR')
+        except KeyError as exc:
+            return ModelAPI.make_json_response({'message': exc.args[0]}, 400, 'MISSING_KEY')
+        except Exception as exc:  # pylint: disable=broad-except
+            logger.exception(exc)
+            return ModelAPI.make_json_response({}, 500, 'UNKNOWN_ERROR')
 
 
 class ModelPrediction(Resource):
@@ -218,9 +219,12 @@ class ModelPrediction(Resource):
             json_data = DatasetPreprocessing.add_time_information(json_data)
             query = pd.DataFrame([json_data])
 
-            x = DatasetPreprocessing.prepare_prediction(modelcfg, query)
+            prepared_query = DatasetPreprocessing.prepare_prediction(
+                modelcfg, query)
 
-            prediction = modelcfg.estimator.predict(x)
+            logger.debug('Predict data columns: %s', prepared_query.columns)
+
+            prediction = modelcfg.estimator.predict(prepared_query)
 
             if modelcfg.dependent_encode:
                 prediction = modelcfg.dependent_encoder.inverse_transform(
@@ -231,18 +235,19 @@ class ModelPrediction(Resource):
 
             result = {
                 'model': modelcfg.config_object(),
+                'preprocessed_query': prepared_query.head(1).to_dict('records'),
                 'prediction': prediction[0]
             }
 
             return ModelAPI.make_json_response(result)
-        except BadRequest as e:
+        except BadRequest:
             return ModelAPI.make_json_response({}, 400, 'BAD_REQUEST')
-        except KeyError as e:
-            return ModelAPI.make_json_response({'message': e.args[0]}, 400, 'MISSING_KEY')
+        except KeyError as exc:
+            return ModelAPI.make_json_response({'message': exc.args[0]}, 400, 'MISSING_KEY')
         except FileNotFoundError:
             return ModelAPI.make_json_response({}, 404, 'NOT_TRAINED')
-        except:
-            print("Unexpected error:", sys.exc_info()[0])
+        except Exception as exc:  # pylint: disable=broad-except
+            logger.exception(exc)
             return ModelAPI.make_json_response({}, 500, 'UNKNOWN_ERROR')
 
     @staticmethod
